
#include <stdio.h>
#include <unistd.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/comms/main.h"
#include "impulse/comms/channel.h"


impulse_bool channel_event_handler(void *data)
{
  impulse_channel channel = (impulse_channel)data;
  impulse_int pending = impulse_channel_pending(channel);
  impulse_int got = 0;
  impulse_char buffer[1024];

  if (pending <= 0)
  {
    impulse_quit();
    return 0;
  }
  
  /* Scale down bytes read to a manageable size */
  if (pending > 1023)
    pending = 1023;
  
  got = impulse_channel_receive(channel, buffer, pending, NULL);
  if (got <= 0)
  {
    impulse_quit();
    return 0;
  }
  
  buffer[got] = '\0';
  printf(buffer);

  return 1;
}


impulse_bool console_event_handler(impulse_event *event,
				   void *data)
{  
  impulse_channel channel = (impulse_channel)data;
  impulse_char buffer[1024];
  impulse_int offset = 0;
  
  for(;;)
  {
    char c;
    
    if (read(event->data.fd.fd, &c, 1) <= 0)
    {
      impulse_quit();
      return 0;
    }
    
    buffer[offset++] = c;
    if (offset == 1022)
    {
      c = '\n';
      buffer[offset++] = c;
    }
    if (c == '\n')
    {
      buffer[offset++] = '\0';
      impulse_channel_send(channel, buffer, strlen(buffer), NULL);
      break;
    }
  }
  return 1;
}



impulse_int main (impulse_int argc,
		  impulse_char *argv[])
{
  impulse_channel channel = NULL;
  impulse_transport transport = NULL;
  impulse_int caps = 0;
  impulse_unistring local;
  impulse_unistring remote;
  impulse_unistring help;
  impulse_unistring arg;
  impulse_command_value value_local = NULL;
  impulse_command_value value_remote = NULL;
  impulse_bool bind = 0;
  
  impulse_init(argc, argv);
  
  local = impulse_unistring_create_utf("local", 8);
  help = impulse_unistring_create_utf("bind to local address [name]", 8);
  arg = impulse_unistring_create_utf("name", 8);
  impulse_command_option_arg(local, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(help);
  
  remote = impulse_unistring_create_utf("remote", 8);
  help = impulse_unistring_create_utf("connect to remote address [name]", 8);
  impulse_command_option_arg(remote, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(help);
  impulse_unistring_destroy(arg);

  impulse_command_parse();
  impulse_comms_init();
  
  if (!impulse_command_set(remote, &value_remote))
    impulse_command_help();
  
  if (impulse_command_set(local, &value_local))
    bind = 1;
  
  impulse_unistring_destroy(local);
  impulse_unistring_destroy(remote);
  
  if (!(transport = impulse_comms_lookup(value_remote->s)))
  {
    impulse_unistring desc = impulse_unistring_create_utf("Unknown transport protocol requested", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value_remote->s);
  }

  caps = IMPULSE_TRANSPORT_CLIENTSERVER | IMPULSE_TRANSPORT_CONNECTION | IMPULSE_TRANSPORT_RELIABLE |
    IMPULSE_TRANSPORT_SEQUENCED;
  
  if ((transport->caps & caps) != caps)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Transport is not capable of reliable client-server channels", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value_remote->s);
  }
  
  channel = impulse_channel_create(transport);
  if (impulse_channel_open(channel, bind ? value_local->s : NULL,
			      value_remote->s, 0) == -1)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot open client channel", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value_remote->s);
  }
  
  impulse_channel_handle(channel, channel_event_handler, channel);
  impulse_event_register_fd(STDIN_FILENO, IMPULSE_EVENT_FD_READ, console_event_handler, NULL, channel);
  
  impulse_main();

  
  impulse_channel_close(channel);
  impulse_channel_destroy(channel);
  
  impulse_comms_cleanup();
  impulse_cleanup();
  
  return 0;
}
