/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * link.c
 * ---------
 *
 * Communication link functions
 */


#include "../../../config.h"

#include "impulse/comms/link.h"
#include "impulse/util/memory.h"
#include "impulse/util/queue.h"


struct __p_impulse_link
{
  impulse_byte *start;
  impulse_byte *middle;
  impulse_byte *end;
  
  impulse_message message;
  
  impulse_queue queue;
};

typedef struct __p_impulse_link *_p_impulse_link;

impulse_link impulse_link_create(impulse_channel channel)
{
  impulse_link chan = impulse_alloc(sizeof(struct _impulse_link));
  _p_impulse_link internal = impulse_alloc(sizeof(struct __p_impulse_link));
  
  internal->start = NULL;
  internal->middle = NULL;
  internal->end = NULL;
  internal->message = NULL;
  internal->queue = impulse_queue_create();
  impulse_queue_cleaner(internal->queue,
			(impulse_queue_cleaner_func)impulse_message_destroy);

  chan->channel = channel;
  chan->internal = internal;
  chan->name = impulse_unistring_create_space(0);
  chan->release = impulse_unistring_create_space(0);
  chan->vendor = impulse_unistring_create_space(0);
  chan->platform = impulse_unistring_create_space(0);
  chan->architecture = impulse_unistring_create_space(0);
  
  return chan;
}


void impulse_link_destroy(impulse_link chan)
{
  _p_impulse_link internal = chan->internal;
    
  impulse_channel_destroy(chan->channel);
  impulse_unistring_destroy(chan->name);
  impulse_unistring_destroy(chan->release);
  impulse_unistring_destroy(chan->vendor);
  impulse_unistring_destroy(chan->platform);
  impulse_unistring_destroy(chan->architecture);
  
  if (internal->start)
    impulse_free(internal->start);
  impulse_queue_destroy(internal->queue);
  
  impulse_free(internal);
  impulse_free(chan);
}


impulse_int _p_impulse_link_process(impulse_link chan,
				    impulse_int amount)
{
  _p_impulse_link internal = chan->internal;
  impulse_int need = internal->end - internal->middle;
  need = (need > amount) ? amount : need;
  
  impulse_channel_receive(chan->channel, internal->middle, need, NULL);
  internal->middle += need;
  
  if (internal->middle == internal->end)
  {
    if (!internal->message) /* We're reading a header */
    {
    }
    else /* We're reading a payload */
    {
    }
  }
  
  return need;
}


impulse_int impulse_link_pending(impulse_link chan)
{
  _p_impulse_link internal = chan->internal;
  impulse_int pending = impulse_channel_pending(chan->channel);
  if (!pending)
    return 0;
  else if (pending < 0)
    ; /* Feck arse */
  else
  {
    /* Make sure we use up all pending data, otherwise select
     * will wastefully spin */
    while (pending)
    {
      impulse_int used;
      used = _p_impulse_link_process(chan, pending);
      pending -= used;
    }
  }
  return impulse_queue_count(internal->queue);
}


impulse_int impulse_link_send(impulse_link chan,
			      impulse_message out)
{
  _p_impulse_link internal = chan->internal;
  impulse_channel_send(chan->channel, out->data, out->data_length, NULL);
  return 0;
}


impulse_int impulse_link_receive(impulse_link chan,
				 impulse_message *in)
{
  _p_impulse_link internal = chan->internal;
  *in = (impulse_message)impulse_queue_serve(internal->queue);
  return 0;
}


/* End of file */
