/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * transport.h
 * -----------
 *
 * Communication transport protocol
 */


#ifndef __impulse_comms_transport_h
#define __impulse_comms_transport_h

#include <impulse/util/unistring.h>
#include <impulse/util/event.h>

/*                            ======--------======                           */

/* 
 * Fix me: define generic error return values
 */

struct _impulse_transport
{
  impulse_unistring name;
  impulse_int caps;
  struct _impulse_transport_funcs *funcs;
};

typedef struct _impulse_transport_funcs *impulse_transport_funcs;
typedef struct _impulse_transport *impulse_transport;
typedef impulse_bool (*impulse_transport_handler)(void *);


/* 
 * Transport capabilities
 */
#define IMPULSE_TRANSPORT_REMOTE         (1 << 0)  /* Capable of comms beyond local machine */
#define IMPULSE_TRANSPORT_RELIABLE       (1 << 1)  /* Guarenteed delivery of data */
#define IMPULSE_TRANSPORT_SEQUENCED      (1 << 2)  /* Sequenced delivery of data */
#define IMPULSE_TRANSPORT_FAST           (1 << 3)  /* Low overhead transmission of data */
#define IMPULSE_TRANSPORT_QUICK          (1 << 4)  /* Low overhead protocol negotiation */
#define IMPULSE_TRANSPORT_STREAM         (1 << 5)  /* Provides a continuous byte stream */
#define IMPULSE_TRANSPORT_DATAGRAM       (1 << 6)  /* Provides a discrete data grams */
#define IMPULSE_TRANSPORT_CONNECTION     (1 << 7)  /* A connection oriented service */
#define IMPULSE_TRANSPORT_CONNECTIONLESS (1 << 8)  /* A connection-less service */
#define IMPULSE_TRANSPORT_CLIENTSERVER   (1 << 9)  /* A client-server system */
#define IMPULSE_TRANSPORT_PEER           (1 << 10) /* A Peer-to-peer architecture */
#define IMPULSE_TRANSPORT_SECURE         (1 << 11) /* Data can not be intercepted */

#endif

/* End of file */
