/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * eventbox.h
 * ----------
 *
 * A box to which events may occur (needs a seperate event delivery system,
 * though)
 */

#ifndef __impulse_rows_eventbox_h
#define __impulse_rows_eventbox_h

#include <impulse/rows/box.h>

/*                            ======--------======                           */

/* EventBox widget type */

typedef void **ROWS_eventbox;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget          ROWS_eventbox_create         (void);
extern ROWS_widget         *ROWS_eventbox_initialise     (int id);
extern inline int           ROWS_eventbox_next_identifier(void);
extern inline ROWS_eventbox ROWS_eventbox_cast           (void **widget);

/*                            ======--------======                           */

typedef impulse_bool (*ROWS_eventbox_func) (void **widget,
					    impulse_int event_num,
					    void *event_data,
					    void *data);

/*                            ======--------======                           */

/* Class specific functions */

extern void ROWS_eventbox_register_widget(ROWS_eventbox box,
					  int widget_level,
					  int max_event);
extern void ROWS_eventbox_register_handler(ROWS_eventbox box,
					   int widget_level,
					   ROWS_eventbox_func func,
					   int  event_number,
					   void *data);
extern void ROWS_eventbox_deregister_handler(ROWS_eventbox box,
					     int widget_level,
					     ROWS_eventbox_func func,
					     int  event_number,
					     void *data);
extern inline void ROWS_eventbox_send(ROWS_eventbox box,
				      int widget_level,
				      void **widget,
				      int event_number,
				      void *event_data);
extern impulse_bool ROWS_eventbox_is_handled(ROWS_eventbox,
					     int widget_level,
					     int event_number);

#endif
