/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * main.h
 * ------
 *
 * General definitions
 */

#ifndef __impulse_rows_main_h
#define __impulse_rows_main_h

#include <impulse/util/error.h>

/*
 * Naming conventions used here
 * 
 * General ROWS functions & types begin with ROWS. Functions specific to
 * an individual widget begin with ROW.
 *
 * Structures and types designed to be internal to the library are prefixed
 * with _p
 *
 * In an alternative header file, abbrieviated names for various functions
 * and widgets may appear.
 */

/*                            ======--------======                           */


/* Main ROWS initialisation / cleanup functions */

void ROWS_init(impulse_int argc,
	       impulse_char *argv[]);
void ROWS_cleanup(void);

/*                            ======--------======                           */

/* Error handling */

enum _ROWS_error {
  ROWS_ERROR_NO_GUI,
  ROWS_ERROR_INTERNAL,

  WIDGET_WRONGTYPE,
  WIDGET_INVALIDTYPE,

  CONTAINER_RECURSIVE,
  CHILD_NOCONTAINER,

  CONSTRUCTOR_GL_LOOP,
  CONSTRUCTOR_GL_INVALID,
  CONSTRUCTOR_GL_NONEXISTANT
};


typedef enum _ROWS_error ROWS_error;

void ROWS_error_raise(impulse_error_level,
		      ROWS_error e,
		      impulse_int value,
		      void *data);

#endif
