/*
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * window.h
 * --------
 *
 * An abstract definition of a window
 */

#ifndef __impulse_rows_window_h
#define __impulse_rows_window_h

#include <impulse/rows/constructor.h>

/*                            ======--------======                           */

/* Window widget type */

typedef void **ROWS_window;

/*                            ======--------======                           */

/* Window attributes */

typedef enum
{
  ROWS_WINDOW_STANDARD = 0,
  ROWS_WINDOW_MENU,
  ROWS_WINDOW_TRANSIENT,
  
  ROWS_WINDOW_COMMS,
  
  ROWS_WINDOW_MAX_TYPE
} ROWS_window_type;

#define ROWS_WINDOW_TITLE       1
#define ROWS_WINDOW_RESIZE      2
#define ROWS_WINDOW_CLOSE       4
#define ROWS_WINDOW_TOGGLESIZE  8
#define ROWS_WINDOW_SCROLLBARS  16
#define ROWS_WINDOW_SLIPPERY    32
#define ROWS_WINDOW_LIGHTWEIGHT 64

typedef impulse_int ROWS_window_attributes;

/*                            ======--------======                           */

/* Window properties */

/*
 * FIXME: Parts that require unimplemented types are commented out
 */

struct _ROWS_window_properties
{
  ROWS_point min_size;

  ROWS_window_type type;
  
  /* ROWS_bitmap shape; */
  /* ROWS_image  bg_image; */
  /* ROWS_colour bg_colour; */

  char *title;
  char *icon_name;
  char *class;

  int own_colourmap;

  ROWS_window_attributes attributes;
};

typedef struct _ROWS_window_properties ROWS_window_properties;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget         ROWS_window_create         (char *title);
extern ROWS_widget        *ROWS_window_initialise     (int   id,
						char *title);
extern inline int          ROWS_window_next_identifier(void);
extern inline ROWS_window  ROWS_window_cast           (void **widget);

/*                            ======--------======                           */

/* Class-specific functions */

extern void ROWS_window_get_properties  (ROWS_window             window,
					 ROWS_window_properties *properties);

extern void ROWS_window_set_title        (ROWS_window  window,
					  char        *title);
extern void ROWS_window_set_class        (ROWS_window  window,
					  char        *class);
extern void ROWS_window_set_icon_name    (ROWS_window  window,
					  char        *icon_name);
/*
extern void ROWS_window_set_icon_image   (ROWS_window  window,
					  ROWS_image   image);
*/
					  
extern void ROWS_window_set_logical_size (ROWS_window  window,
					  ROWS_point  *size);
extern void ROWS_window_set_minimum_size (ROWS_window  window,
					  ROWS_point  *size);
extern void ROWS_window_set_physical_size(ROWS_window  window,
					  ROWS_point  *size);
extern void ROWS_window_set_size_steps   (ROWS_window  window,
					  ROWS_point  *steps);
extern void ROWS_window_set_current_pos  (ROWS_window  window,
					  ROWS_point  *position);
/*
extern void ROWS_window_set_shape        (ROWS_window  window,
					  ROWS_bitmap  bitmap);
*/
					  
extern void ROWS_window_set_attributes   (ROWS_window            window,
					  ROWS_window_attributes attributes);

extern void ROWS_window_set_bg_colour    (ROWS_window window,
					  ROWS_colour colour);
/*
extern void ROWS_window_set_bg_image     (ROWS_window window,
					  ROWS_image  image);
*/

extern void ROWS_window_set_type        (ROWS_window      window,
					 ROWS_window_type type);
extern void ROWS_window_own_colourmap   (ROWS_window      window);

#endif
