/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * measure.h
 * ---------
 *
 * Measuring things
 */

#ifndef __impulse_util_measure_h
#define __impulse_util_measure_h

#include <impulse/util/setup.h>

#ifdef __cplusplus
extern "C" {  
#endif

#define IMPULSE_DOUBLE_PRECISION 1

#if IMPULSE_DOUBLE_PRECISION
typedef impulse_double microns; /* Base measurement (micrometers) */
#else
typedef impulse_int  microns;
#endif

#define PIXEL_DPM  3543.0    /* Dots per meter. 90 DPI...    */
#define HALF_PIXEL 141.1111  /* Constant used in some places */
#define PIXEL_SIZE 282.2222  /* Size of a pixel in microns   */

/* FIXME: are there more accurate conversions available? */

#define IN_TO_MICRONS(x)    ((microns) (x)*2540000)
#define MM_TO_MICRONS(x)    ((microns) (x)*1000)
#define CM_TO_MICRONS(x)    ((microns) (x)*100000)
#define PIXEL_TO_MICRONS(x) ((microns) (x)*PIXEL_SIZE)

#define MICRONS_TO_PIXEL(x) ((impulse_int)   (((x)+HALF_PIXEL)/PIXEL_SIZE))
#define MICRONS_TO_IN(x)    ((double)  (x)/2540000.0)
#define MICRONS_TO_MM(x)    ((double)  (x)/1000.0)
#define MICRONS_TO_CM(x)    ((double)  (x)/100000.0)

#define PIXEL_MULTIPLIER    ((double) PIXEL_SIZE)

#ifdef __cplusplus
}
#endif

#endif
