/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * sparse.h
 * --------
 *
 * Sparse arrays
 */

#ifndef __impulse_util_sparse_h
#define __impulse_util_sparse_h

#include <impulse/util/iterator.h>

#ifdef __cplusplus
extern "C" {  
#endif

/*                            ======--------======                           */

typedef struct _impulse_sparse *impulse_sparse;
typedef void (*impulse_sparse_cleaner_func)(void *);

/*                            ======--------======                           */

/* Sparse array functions */

impulse_sparse impulse_sparse_create(void);
impulse_sparse impulse_sparse_create_gran(impulse_uint granularity);
void impulse_sparse_destroy(impulse_sparse array);

void impulse_sparse_cleaner(impulse_sparse array,
			    impulse_sparse_cleaner_func func);
impulse_iterator impulse_sparse_elements(impulse_sparse array);
impulse_iterator impulse_sparse_keys(impulse_sparse array);

impulse_bool impulse_sparse_empty(impulse_sparse array);
impulse_uint impulse_sparse_count(impulse_sparse array);

void impulse_sparse_add(impulse_sparse array,
			impulse_uint key,
			void *data);
void *impulse_sparse_recover(impulse_sparse array,
			     impulse_uint key);
void *impulse_sparse_remove(impulse_sparse array,
			    impulse_uint key);
void impulse_sparse_clear(impulse_sparse array);
impulse_bool impulse_sparse_contains(impulse_sparse array,
				     void *data);
impulse_bool impulse_sparse_contains_key(impulse_sparse array,
					 impulse_uint key);


#ifdef __cplusplus
}
#endif

#endif

